#include<stdlib.h>
#include<stdio.h>

typedef struct
{
  unsigned int max_taille; /* cases allouees */
  unsigned int taille;     /* cases utilisees */
  int * tab;
} dyntab;

void init_dyntab(dyntab *pt, unsigned int init_taille)
{
  pt->max_taille = init_taille;
  pt->taille = 0;
  pt->tab = (int*) malloc(init_taille*sizeof(int));
  if (pt->tab == NULL)
    {
      printf("Plus de memoire\n");
      exit(1);
    }
}

void realloc_dyntab(dyntab *pt)
{
  int nouvelle_taille = pt->max_taille*2;
  pt->tab = (int *) realloc(pt->tab, nouvelle_taille);
  pt->max_taille = nouvelle_taille;
}


void free_dyntab(dyntab *pt)
{
  free(pt->tab);
  pt->max_taille = 0;
  pt->taille = 0;
  pt->tab = NULL;
}

void insere_fin(dyntab *pt, int e)
/* Complexite O(1) */
{
  if (pt->taille == pt->max_taille) realloc_dyntab(pt);
  pt->tab[pt->taille] = e;
  pt->taille++;
}

void insere_position(dyntab *pt, int e, int pos)
/* Complexite O(taille) */
{
  int i;
  if (pt->taille == pt->max_taille) realloc_dyntab(pt);

  /* boucle sur la case d'arrivee du decalage */
  for (i = pt->taille; i > pos; i--) pt->tab[i] = pt->tab[i-1];

  /* boucle sur la case de depart du decalage */
  // for(i = pt->taille -1; i>= pos; i--) pt->tab[i+1] = pt->tab[i];

  pt->tab[pos] = e;
  pt->taille++;
}

void insere_debut(dyntab *pt, int e)
/* Complexite O(taille) */
{
  int i;
  if (pt->taille == pt->max_taille) realloc_dyntab(pt);

  /* boucle sur la case d'arrivee du decalage */
  for (i = pt->taille; i > 0; i--) pt->tab[i] = pt->tab[i-1];
  pt->tab[0] = e;
  pt->taille++;
}

void supprime_fin(dyntab *pt)
/* Complexite O(1) */
{
  pt->taille--;
}


void supprime_position(dyntab *pt, int pos)
/* Complexite O(taille) */
{
  int i;
  for (i = pos; i < pt->taille-1; i++) pt->tab[i] = pt->tab[i+1];
  pt->taille--;
}


void supprime_debut(dyntab *pt)
/* Complexite O(taille) */
{
  int i;
  for (i = 0; i < pt->taille-1; i++) pt->tab[i] = pt->tab[i+1];
  pt->taille--;
}
